// LED Checker v3, subroutine
// for ATmega8L, 8MHz
// [22/05/19] Created rev_c1815@yahoo.co.jp

#include "main.h"
#include "lcd.h"

#include <avr/eeprom.h>
#include <avr/pgmspace.h>

struct sPreference g_preference;
#define EEP_START_ADDR 0x10
extern uint16_t g_adcResult[2];


void LoadEEPSetting(struct sPreference* pSetting)
{
	uint8_t signature = eeprom_read_byte((uint8_t*)EEP_START_ADDR);
	if(signature == 0x55){
		eeprom_read_block(pSetting, (uint8_t*)EEP_START_ADDR+1, sizeof(struct sPreference));
	}
	if((pSetting->timer1_max < 300) || (pSetting->timer1_max > 700)) pSetting->timer1_max = 500;
}
void SaveEEPSetting(const struct sPreference* pSetting)
{
	// signature
	eeprom_write_byte((uint8_t*)EEP_START_ADDR, 0x55);
	eeprom_write_block(pSetting, (uint8_t*)EEP_START_ADDR+1, sizeof(struct sPreference));
}



void CalibrationRoutine(uint8_t swState, struct sPreference* pPreference)
{
	if(swState == SW_SW5) // CUR+
	{
		if(pPreference->timer1_max < 700) pPreference->timer1_max ++;
	}
	else if(swState == SW_SW4) // CUR-
	{
		if(pPreference->timer1_max < 300) pPreference->timer1_max --;
	}
	else if(swState == (SW_SW6 | SW_LONG)) // Power
	{
		// Store to SPI-ROM
		Shutdown();
	}
	
}

const uint8_t curList[] = {
	1,2,3,4,5,6,7,8,9, 
	10,12,15,18,22,27,33,39,47,56,68,82, 
	100,110,120,130,150,180,200,220,240}; // x0.1mA
const uint8_t dutyList[] = {
	// LCM(1,2,4,6,8,12,16) = 48
	1, 2,4,6,8,12,16
	// 1/6: 42.67/256, 1/12 = 21.33/256
}; // 1/N

#define LED_NC_THRESHOLD 1000 // if dac result is below this level, display as LED is Not Connected
void LCDShowVoltage(uint16_t voltage)
{
	if(LED_NC_THRESHOLD && (g_adcResult[1] > LED_NC_THRESHOLD))
	{
		LCD_Print("(NC)   ", 7);
	}
	else
	{
		LCD_Putc( '0' + ((voltage/100)%10));
		LCD_Putc( '.');
		LCD_Putc( '0' + ((voltage/10)%10));
		LCD_Putc( '0' + ((voltage)%10));
		LCD_Print("V  ", 3);
	}
}
void LCDShowResistor(uint16_t resistor, uint8_t vdd) // ohm
{
	if(LED_NC_THRESHOLD && (g_adcResult[1] > LED_NC_THRESHOLD))
	{
		LCD_Print(" (NC)   ", 8);
	}
	else if(vdd == 3)
	{
		LCD_Putc(0x00); // 3.
		LCD_Putc(0x01); // 3v
	}
	else
	{
		LCD_Putc(' ');
		LCD_Putc(0x02); // 5v
	}
	if(resistor >= 10000){
		LCD_Putc('0' + resistor/10000);
		LCD_Putc('0' + (resistor/1000)%10);
		LCD_Putc('k');
	}
	else if(resistor >= 1000){
		LCD_Putc('0' + resistor/1000);
		LCD_Putc('k');
		LCD_Putc('0' + (resistor/100)%10);
	}else{
		LCD_PrintBCD(resistor);
	}
	LCD_Putc(0xF4); // ohm
	LCD_Putc(' ');
}

void LCDMessageUpdate(const struct sPreference* pPreference)
{
	// Update Message (16x1)
	uint16_t voltage = (uint32_t)(g_adcResult[1] - g_adcResult[0]) * 500 / 1024; // x10mV
	uint16_t current = curList[pPreference->current]; // x0.1mA
	uint8_t  duty    = dutyList[pPreference->duty];
	if(DUTY_CHANGE_METHOD) current *= duty;
	
	// debug mode
	if(0){
		LCD_SetTextPos(0);
		LCD_PrintBCD(g_adcResult[0]);
		LCD_Putc( ',');
		LCD_PrintBCD(g_adcResult[1]);
		LCD_Putc( ' ');
		LCD_Putc( ' ');
		return;
	}
	
	if(pPreference->display == 0)
	{
		// 1234567890123456
		// XX.XmA X.XXV 1/N
		LCD_SetTextPos(0);
		LCD_Putc((current/100)? ('0' + (current/100)): ' ');
		LCD_Putc( '0' + ((current/10)%10));
		LCD_Putc( '.');
		LCD_Putc( '0' + (current%10));
		LCD_Print("mA ", 3);
		
		LCDShowVoltage(voltage);
		
		LCD_SetTextPos(0x40 | (14-8));
		LCD_Putc(0x03); // 1/
		if(duty < 10) LCD_Putc('0'+duty);
		else if(duty == 12) LCD_Putc(0x04);
		else if(duty == 16) LCD_Putc(0x05);
		else LCD_Putc('?');
	}
	else if(pPreference->display == 1)
	{
		uint16_t resistor = (3300-voltage*10)*10/current; // ohm
//		if(resistor > 9999) resistor = 9999;
		// 1234567890123456
		// XX.XmAvvxxxZ 1/N
		LCD_SetTextPos(0);
		LCD_Putc((current/100)? ('0' + (current/100)): ' ');
		LCD_Putc( '0' + ((current/10)%10));
		LCD_Putc( '.');
		LCD_Putc( '0' + (current%10));
		LCD_Print("mA", 2);
		
		LCDShowResistor(resistor, 3);
		LCD_SetTextPos(0x40 | (14-8));
		
		LCD_Putc(0x03); // 1/
		if(duty < 10) LCD_Putc('0'+duty);
		else if(duty == 12) LCD_Putc(0x04);
		else if(duty == 16) LCD_Putc(0x05);
		else LCD_Putc('?');
	}
	else
	{
		uint16_t resistor = (5000-voltage*10)*10/current; // ohm
//		if(resistor > 9999) resistor = 9999;
		// 1234567890123456
		// XX.XmA vxxxZ 1/N
		LCD_SetTextPos(0);
		LCD_Putc((current/100)? ('0' + (current/100)): ' ');
		LCD_Putc( '0' + ((current/10)%10));
		LCD_Putc( '.');
		LCD_Putc( '0' + (current%10));
		LCD_Print("mA", 2);

		LCDShowResistor(resistor, 5);
		LCD_SetTextPos(0x40 | (14-8));
		
		LCD_Putc(0x03); // 1/
		if(duty < 10) LCD_Putc('0'+duty);
		else if(duty == 12) LCD_Putc(0x04);
		else if(duty == 16) LCD_Putc(0x05);
		else LCD_Putc('?');
	}
}
void UpdatePWMTimer(const struct sPreference* pPreference)
{
	if(DUTY_CHANGE_METHOD == 1) // multiply current if available
	{
		uint16_t current_duty = (uint16_t)curList[pPreference->current] * dutyList[pPreference->duty];
		if(current_duty >= ICR1) current_duty = ICR1;
		OCR1A = current_duty;
	}else{
		OCR1A = curList[pPreference->current];
	}
	
	if(pPreference->duty == 0) // 100%
	{
		// PWM OFF and output H/L
		TCCR2 &= ~0b00100000;
	}
	else
	{
		TCCR2 |= 0b00100000;
		OCR2 = 256/dutyList[pPreference->duty]-1;
	}
}
void Shutdown(void)
{
	LCD_SetTextPos(0);
	// Store to SPI-ROM
	SaveEEPSetting(&g_preference);
	// SHDN_CTL = 0
	PORTB &= ~(1<<6);
	LCD_Print("SHDN", 4);
	while(1);
}
uint8_t IsCurrentAvailable(uint8_t current, uint8_t duty)
{
	if(DUTY_CHANGE_METHOD == 0) return 1;
	
	uint8_t actualCurrent = (uint16_t)curList[current] * dutyList[duty]; // x0.1mA
	if(actualCurrent > ICR1) return 0;
	
	return 1;
}
void SwitchRoutine(uint8_t swState, struct sPreference* pPreference)
{
	if((swState & 0xF) == SW_SW5) // CUR+
	{
		if(pPreference->current < sizeof(curList)/sizeof(curList[0])-1) {
			if(IsCurrentAvailable(pPreference->current+1, pPreference->duty)){
				pPreference->current ++;
			}
		}
		// Update Current PWM
		UpdatePWMTimer(pPreference);
	}
	else if((swState & 0xF) == SW_SW4) // CUR-
	{
		if(pPreference->current >0) pPreference->current --;
		UpdatePWMTimer(pPreference);
	}
	else if(swState == SW_SW3) // DUTY+
	{
		if(pPreference->duty < sizeof(dutyList)/sizeof(dutyList[0])-1) {
			if(IsCurrentAvailable(pPreference->current, pPreference->duty+1)){
				pPreference->duty ++;
			}
		}
		UpdatePWMTimer(pPreference);
	}
	else if(swState == SW_SW2) // DUTY-
	{
		if(pPreference->duty > 0) pPreference->duty --;
		UpdatePWMTimer(pPreference);
	}
	else if(swState == SW_SW1) // Display
	{
		pPreference->display = (pPreference->display+1)%3;
	}
	else if(swState == (SW_SW6 | SW_LONG)) // Power
	{
		Shutdown();
	}
}
